#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"
#include <curl/curl.h>
#include <sstream>
#include <string>

struct Input{
    std::vector<double> numbers;
    size_t bin_count{};
};

Input input_data(std::istream& in, bool prompt) {
    size_t number_count;
    if (prompt){
        std::cerr << "Enter number count: ";
    }
    in >> number_count;

    if (prompt){
        std::cerr << "Enter numbers: \n";
    }

    Input result;
    result.numbers.resize(number_count);
    for (size_t i = 0; i < number_count; ++i) {
        in >> result.numbers[i];
    }

    if (prompt){
        std::cerr << "Enter bucket: ";
    }

    in >> result.bin_count;

    return result;
};
size_t write_data(void*items,size_t item_size, size_t item_count, void* ctx){
    size_t data_size = item_size * item_count;
    std::stringstream* buffer = static_cast<std::stringstream*>(ctx);
     buffer->write(static_cast<char*>(items), data_size);
    return data_size;

}
Input download(const std::string&address){
    std::stringstream buffer;
    CURL* curl = curl_easy_init();
    if(curl) {

            CURLcode res;
            curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
            curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
            res = curl_easy_perform(curl);

            if (res != CURLE_OK){
                std::cerr << curl_easy_strerror(res);
                curl_easy_cleanup(curl);
                exit(1);
            }
            curl_easy_cleanup(curl);
        }
    return input_data(buffer, false);
}

using namespace std;

int main(int argc, char* argv[]) {
    Input input;
    if (argc > 1) {
        input = download(argv[1]);
    } else {
        input = input_data(cin, true);
    }

    auto bins = make_histogram(input.numbers, input.bin_count);
    show_histogram_svg(bins);
    return 0;
}
